import simplejson as json


class HSProperty:

    def __init__(self, name, **kwargs):

        self.name = name.replace(" ", "_").tolower()
        self.label = kwargs.get("label", name)
        self.description = kwargs.get("description")
        self.groupName = kwargs.get("groupName", "contactinformation")
        self.type = kwargs.get("type", "string")
        self.fieldType = kwargs.get("fieldType")
        self.formField = kwargs.get("formField", True)
        self.displayOrder = kwargs.get("displayOrder", 6),
        self.options = kwargs.get("options", [])

        if not self.fieldType:
            if self.type == "string":
                self.fieldType = "text"
            elif self.type == "number":
                self.fieldType = "number"
            elif self.type == "bool":
                self.property_type = "enumeration"
                self.fieldType = "booleancheckbox"
                self.options = [
                    {
                        "displayOrder": -1,
                        "label": "Yes",
                        "value": "true"
                    },
                    {
                        "displayOrder": -1,
                        "label": "No",
                        "value": "false"
                    }
                ]
            elif self.type == "datetime":
                self.fieldType = "date"
            elif self.type == "enumeration":
                self.fieldType = "checkbox"

    def update(self, **kwargs):
        for item, value in kwargs.iteritems():
            setattr(self, item, value)

    def add_option(self, option, value):
        new_option = {
            "label": option,
            "value": value,
        }
        self.options.append(new_option)

    def add_options(self, options):
        for option in options:
            self.add_option(option=option[0], value=option[1])

    def remove_option(self, target):
        self.options = [option for option in self.options if option["label"] != target]

    def remove_options(self, targets):
        self.options = [option for option in self.options if option["label"] not in targets]

    def to_json(self):
        json_data = {
            "name": self.name,
            "label": self.label,
            "description": self.description,
            "groupName": self.groupName,
            "displayOrder": self.displayOrder,
            "formField": self.formField,
            "type": self.type,
            "fieldType": self.fieldType,
            "options": self.options,
        }
        return json.dumps(json_data)

    @classmethod
    def from_json(cls, data):
        hsproperty = HSProperty()
        try:
            hsproperty.name = data["name"]
            hsproperty.label = data["label"]
            hsproperty.description = data["description"]
            hsproperty.groupName = data["groupName"]
            hsproperty.displayOrder = data["displayOrder"]
            hsproperty.type = data["type"]
            hsproperty.fieldType = data["fieldType"]
            hsproperty.options = data["options"]
        except Exception as exc:
            raise SyntaxError(
                "There is an error in the property data you submitted to this function. "
                "Expected to find a fully formed property object, and instead found missing data. "
                "Refer to the original exception message for details: {exc}".format(exc=exc)
            )
        else:
            return hsproperty



class HAPIError(Exception):
    def __init__(self, message, response, code):
        self.value = self.message = message
        self.resposne = response
        self.code = code

    def __str__(self):
        return "{msg} -- HubSpot API responded with: {response} - code {code}".format(
            msg=self.message,
            response=self.response,
            code=self.code
        )

class HSApi:

    def __init__(self, parent):
        self.credentials = parent

    def add_authentication(self, params):
        params[self.credentials.authorization] = self.credentials.auth_value

        return params



import simplejson as json
import requests
import math
from .hsapi import HSApi, HAPIError


class Files(HSApi):

    BASE_URL = "https://api.hubapi.com/content/api/v2/files"
    FOLDERS = "https://api.hubapi.com/content/api/v2/folders"

    def upload_new_file(self, filename, folders=None, overwrite="false"):
        params = self.add_authentication({})

        data = {"overwrite": overwrite}

        if folders is not None:
            data['folder_paths'] = folders

        files = {"files": open(filename, 'rb')}

        r = requests.post(self.BASE_URL, params=params, data=data, files=files)

        if r.status_code in [200, 201]:
            response = r.json()
            return response['objects'][0]
        else:
            raise HAPIError(
                message="New File Post Failed",
                response=r.text,
                code=r.status_code
            )

    def delete_all(self, type="files"):

        all_meta = self.get_all_metadata(type=type)

        all_deleted = True

        base_url = self.BASE_URL if type == "files" else self. FOLDERS

        for file in all_meta:
            url = base_url + "/{fileid}".format(fileid=file["id"])

            r = requests.delete(url, params=self.add_authentication({}))

            if r.status_code in range(200, 299):
                continue

            all_deleted = False

        return all_deleted


    def get_all_metadata(self, type="files", params=None):

        allowed_params = [
            "alt_key",
            "archived",
            "created",
            "deleted_at",
            "extension",
            "folder_id",
            "id",
            "name",
            "type",
            "parent_folder_id"
        ]

        if params is not None:
            params = {key: value for (key, value) in params.items() if any(key.startswith(param) for param in allowed_params)}
        else:
            params = {}

        params['limit'] = 20

        metadata = []

        url = self.BASE_URL if type == "files" else self.FOLDERS

        total = self.get_total(url, params)

        for page in range(1, (math.ceil(total / 20) + 1)):
            metadata.extend(self.get_page(url, page, params))

        return metadata

    def get_total(self, url, params):

        params = self.add_authentication(params)
        r = requests.get(url, params=params)

        if r.status_code == 200:
            response = r.json()
            return response["total_count"]
        else:
            raise HAPIError(
                message="Get Total Files failed",
                response=r.text,
                code=r.status_code
            )

    def get_page(self, url, page, params):

        params['offset'] = params['limit'] * (page - 1)

        params = self.add_authentication(params)

        r = requests.get(url, params=params)

        if r.status_code == 200:
            response = r.json()
            return response["objects"]
        else:
            raise HAPIError(
                message="Get page failed",
                response=r.text,
                code=r.status_code
            )

    def get_folder_path(self, id):

        if id is None:
            return None

        url = self.FOLDERS + "/{id}/".format(id=id)

        params = self.add_authentication({})
        r = requests.get(url, params=params)

        if r.status_code == 200:
            response = r.json()
            return response["full_path"]
        else:
            raise HAPIError(
                message="Get folder path failed",
                response=r.text,
                code=r.status_code
            )

    def create_folder(self, name, parent_id=None):

        data = {"name": name}

        if parent_id is not None:
            data["parent_folder_id"] = parent_id

        r = requests.post(self.FOLDERS, params=self.add_authentication({}), data=json.dumps(data))

        if r.status_code in range(200, 300):
            response = r.json()
            return response


        return False


import simplejson as json
import requests
from .hsapi import HSApi, HAPIError
from .utilities import HSProperty


class Properties(HSApi):
    BASE_URL = "https://api.hubapi.com/contacts/v1/"
    PROPERTIES_URL = BASE_URL + "properties/"
    GROUPS_URL = BASE_URL + "groups/"

    @classmethod
    def get_name(self):
        return "Properties"

    def get_all(self):
        params = self.add_authentication({})
        r = requests.get(self.PROPERTIES_URL, params=params)
        if r.status_code == 200:
            return r.json()

        raise HAPIError(
            message="Properties Get All Method Failed",
            response=r.text,
            code=r.status_code
        )

    def get_by_name(self, name):
        r = requests.get(self.PROPERTIES_URL + name, params = self.add_authentication({}))

        if r.status_code == 200:
            return r.json()

        raise HAPIError(
            message="Properties Get by name Method Failed",
            response=r.text,
            code=r.status_code
        )

    def create_property(self, hsproperty=None, **kwargs):
        if not isinstance(hsproperty, HSProperty):
            name = kwargs.get("name")
            if not name:
                raise SyntaxError("You must supply a name value")
            hsproperty = HSProperty(name=name, **kwargs)

        r = requests.put(
            self.PROPERTIES_URL + hsproperty.name,
            params=self.add_authentication({}),
            data=hsproperty.to_json(),
            headers={'content-type': 'application/json'}
        )
        if r.status_code is 200:
            return True
        else:
            return False

    def update_property(self, name, data):

        url = self.PROPERTIES_URL + name

        r = requests.post(
            url,
            params=self.add_authentication({}),
            data=data,
            headers={'content-type': 'application/json'}
        )

        if r.status_code is 200:
            return True

        raise HAPIError(
            message="Properties Update Method Failed",
            response=r.text,
            code=r.status_code
        )

    def delete_property(self, name):
        url = self.PROPERTIES_URL + name
        r = requests.delete(
            url,
            params=self.add_authentication({})
        )

        if r.status_code is 204:
            return True
        else:
            return False

    def get_property_group(self, name):
        url = self.GROUPS_URL + name

        r = requests.get(
            url,
            params=self.add_authentication({}),
        )

        if r.status_code == 200:
            return r.json()
        if r.status_code == 404:
            return None
        else:
            return False

    def create_property_group(self, name, **kwargs):
        hs_property = {
            "name": name.replace(" ", "_"),
            "displayName": kwargs.get("displayName", name),
            "displayOrder": kwargs.get("displayOrder", 5)
        }

        url = self.GROUPS_URL + hs_property["name"]

        r = requests.put(
            url,
            params=self.add_authentication({}),
            data=json.dumps(hs_property),
            headers={'content-type': 'application/json'}
        )
        if r.status_code in [200, 409]:
            return True
        else:
            return False

    def update_property_group(self, name, **kwargs):
        hs_property = {
            "name": name.replace(" ", "_"),
            "displayName": kwargs.get("displayName"),
            "displayOrder": kwargs.get("displayOrder")
        }

        hs_property = dict((k, v) for k, v in hs_property.iteritems() if v)

        url = self.GROUPS_URL + hs_property["name"]

        r = requests.put(
            url,
            params=self.add_authentication({}),
            data=json.dumps(hs_property),
            headers={'content-type': 'application/json'}
        )

        if r.status_code is 200:
            return True
        else:
            return False

    def delete_property_group(self, name):
        url = self.GROUPS_URL + name
        r = requests.delete(
            url,
            params=self.add_authentication({})
        )

        if r.status_code is 204:
            return True
        else:
            return False



import simplejson as json
import requests
from .hsapi import HSApi


class Contacts(HSApi):
    BASE_URL = "https://api.hubapi.com/contacts/v1/contact/"

    @classmethod
    def get_name(self):
        return "Contacts"

    def get_contact_by_email(self, email, **kwargs):
        url = self.BASE_URL + "/email/{email}/profile".format(email=email)
        params = self.add_authentication({})
        r = requests.get(url, params=params)

        if r.status_code == 200:
            return r.json()
        else:
            return r.text

    def update_contact(self, contact_id, fields):
        headers = {'content-type': 'application/json'}

        #Set id to VID if arg was a string
        if isinstance(contact_id, basestring):
            contact_id = self.get_contact_by_email(contact_id)["vid"]

        url = self.BASE_URL + "vid/{vid}/profile".format(contact_id)

        if not isinstance(fields, dict):
            raise SyntaxError("Fields should be a dictionary")

        properties = []

        """
        Convert the dictionary passed in as fields into properties
        """
        for field, value in fields.iteritems():
            properties.append({"property": field, "value": value})

        properties = {"properties": properties}

        params = self.add_authentication({})

        r = requests.post(
            url,
            data=json.dumps(properties),
            params=params,
            headers=headers
        )

        if r.status_code is 204:
            return True
        else:
            return r.text

    def batch_update(self, contact_batch):
        headers = {'content-type': 'application/json'}

        url = self.BASE_URL + "batch"
        
        batch = []

        for contact in contact_batch:
            if isinstance(contact, dict): 
                property_list = []               
                for field, value in contact.items():
                    property_list.append({"property": field, "value": value})
                batch.append({"email": contact.get('email'), "properties": property_list})
                del(property_list)
            else:
                raise SyntaxError("Contact should be a dictionary")

        params = self.add_authentication({})

        r = requests.post(
            url,
            data=json.dumps(batch),
            params=params,
            headers=headers
        )

        if r.status_code is 204:
            return True
        else:
            return r.text


from hubspot.hsapi import HSApi
import requests


class Blog:

    def __init__(self, parent):
        self.cms = CMS(parent)
        self.cos = COS(parent)


class CMS(HSApi):
    URL = "https://api.hubapi.com/blog/v1"

    def get_posts(self, guid):
        url = self.URL + "/{guid}/posts.json".format(guid=guid)

        params = self.add_authentication({})

        r = requests.get(url, params=params)

        if r.status_code is 200:
            return r.json()
        else:
            return False

    def get_blog_list(self):
        url = self.URL + "/list.json"

        params = self.add_authentication({})

        r = requests.get(url, params=params)

        if r.status_code is 200:
            return r.json()
        else:
            return False

    @staticmethod
    def to_cos(data, target_portal):
        author_id = target_portal.blog.cos.get_author_id(data['authorDisplayName'])
        translated_data = {
            "blog_author_id": author_id,
            "is_draft": data['draft'],
            "meta_description": data['metaDescription'],
            "meta_keywords": data['metaKeywords'],
            "name": data['title'],
            "post_body": data['body'],
            "post_summary": data['summary'],
            "publish_date": data['publishTimestamp'],
        }
        translated_data = dict((k, v) for k, v in translated_data.items() if v)
        return translated_data


class COS(HSApi):
    URL = "https://api.hubapi.com/blog/v2"

    def post(self, data):
        url = self.URL + "/blog-posts"

        params = self.add_authentication({})

        r = requests.put(url, params=params, data=data)

        if r.status_code is 200:
            return True
        else:
            return False

    def get_author_id(self, name):
        url = self.URL + "/blog-authors"

        params = self.add_authentication({"full_name": name})

        r = requests.get(url, params=params)

        if r.status_code is 200:
            return r.json()['objects'][0]['id']
        else:
            return False


import simplejson as json
import requests
from .hsapi import HSApi


class Forms(HSApi):

    BASE_URL = "https://api.hubapi.com/contacts/v1/forms"

    def submit(self, guid, fields, context):

        if not isinstance(fields, dict):
            raise SyntaxError("Field Data should be in a dictionary format")

        if not isinstance(context, dict):
            raise SyntaxError("Context data should be in dictionary format")
        elif not fields.get("email"):
            if not (context.get("hutk") or context.get("ipAddress")):
                raise SyntaxError(
                    "Not enough data to identify a lead, please include an email address"
                    " in the field dictionary or a hubspot user token"
                )
        fields['hs_context'] = json.dumps(context)

        url = "https://forms.hubspot.com/uploads/form/v2/{portalid}/{formid}"

        r = requests.post(url.format(
            portalid=self.credentials.portal_id,
            formid=guid),
            data=fields,
        )

        if r.status_code == 204:
            return True
        else:
            return r.text

    def create(self, name, fields, **kwargs):
        if not isinstance(fields, dict):
            raise SyntaxError("Field Data should be in a dictionary format")

        properties = self.credentials.properties
        property_list = properties.get_all()
        form_fields = []

        for field, options in fields.items():
            prop = properties.get(field, target=property_list)
            if prop is not False:
                form_fields.append(prop)
            else:
                options["name"] = field
                form_fields.append(options)

        form_data = {
            "name": name,
            "action": "",
            "method": "POST",
            "cssClass": "hs-form stacked",
            "redirect": "",
            "submitText": "Sign Up",
            "followUpId": "",
            "leadNurturingCampaignId": "",
            "notifyRecipients": "",
            "embeddedCode": "",
            "fields": form_fields
        }

        for key, value in kwargs.items():
            form_data[key] = value

        params = self.add_authentication({})
        r = requests.post(self.BASE_URL, params=params, data=json.dumps(form_data))

        if r.status_code == 200:
            return r.json()["guid"]
        elif r.status_code == 409:
            return r.json()
        else:
            raise SyntaxError(
                "Create form post request responded with {code} {text}".format(
                    code=r.status_code,
                    text=r.text
                )
            )

    def get(self, guid):
        url = self.BASE_URL + "/{guid}".format(guid=guid)

        params = self.add_authentication({})

        r = requests.get(url, params=params)

        if r.status_code == 200:
            return r.json()
        else:
            return False



from .forms import Forms
from .contacts import Contacts
from .properties import Properties
from .files import Files
from .blog import Blog
import requests
from delorean import Delorean


class HubSpot:

    def __init__(self, **kwargs):
        self.client_secret = kwargs.get("client_secret")
        self.client_id = kwargs.get("client_id")
        self.app = False
        self.authorization = "hapikey"
        self.portal_id = kwargs.get("portal_id")

        if self.client_id is not None or self.client_secret is not None:
            if self.client_id is None or self.client_secret is None:
                raise SyntaxError(
                    "Both Client_id and client_secret are required for HubSpot app initilization"
                )
            self.app = True
            self.authorization = "access_token"
            self.refresh_token = kwargs.get("refresh_token")

            if self.refresh_token is None:
                raise SyntaxError(
                    "You must provide a refresh_token along with access_token"
                )

        self.auth_value = kwargs.get(self.authorization)

        if self.auth_value is None:
            raise SyntaxError(
                "You must include a hapi_key or access_token attribute when initalizing the API"
            )

        if self.portal_id is None:
            raise SyntaxError(
                "You did not provide the portal_id required parameter."
            )

        self.forms = Forms(self)
        self.contacts = Contacts(self)
        self.properties = Properties(self)
        self.files = Files(self)
        self.blog = Blog(self)

    def get_new_token(self):
        """
        Check if the access token has expired. If it has, use the refresh token
        to revalidate the portal.
        """

        url = "https://api.hubapi.com/auth/v1/refresh"

        data = {
            "refresh_token": self.refresh_token,
            "client_id": self.client_id,
            "grant_type": "refresh_token"
        }

        r = requests.post(url, data=data)

        if r.status_code != 200:
            return False

        data = r.json()

        self.access_token = data["access_token"]
        self.refresh_token = data["refresh_token"]

        params = {
            "access_token": self.access_token,
            "refresh_token": self.refresh_token,
            "expires_at": int(data["expires_in"]) + int(Delorean().epoch())
        }

        return params



from distutils.core import setup

setup(
    name='hubspot',
    version='0.1.13dev',
    packages=['hubspot',],
    license='LICENSE.txt',
    long_description=open('README.txt').read(), requires=['requests']
)
